#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "bc.h"

#include "report.h"

#include "callbacks.h"
#include "interface.h"
#include "support.h"

void
on_button_browse_clicked (GtkButton * button, gpointer user_data)
{
	openfile ((GtkWidget *) button);
}

void
openfile (GtkWidget * caller)
{
	GtkWidget *filechooserdialog1=NULL;

	filechooserdialog1 = gtk_file_chooser_dialog_new ("Open File", NULL,
							  GTK_FILE_CHOOSER_ACTION_OPEN,
							  GTK_STOCK_CANCEL,
							  GTK_RESPONSE_CANCEL,
							  GTK_STOCK_OPEN,
							  GTK_RESPONSE_ACCEPT,
							  NULL);

	if (gtk_dialog_run (GTK_DIALOG (filechooserdialog1)) ==
	    GTK_RESPONSE_ACCEPT)
	{
		char *filename;

		filename =
			gtk_file_chooser_get_filename (GTK_FILE_CHOOSER
						       (filechooserdialog1));

		gtk_entry_set_text (GTK_ENTRY (lookup_widget (caller,
							      "entry_report_filename")),
				    filename);

		g_free (filename);

	}

	gtk_widget_destroy (filechooserdialog1);
}

void
on_window1_destroy (GtkObject * object, gpointer user_data)
{
	gtk_main_quit ();
}


void
on_open1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	openfile (GTK_WIDGET (menuitem));
}


void
on_about1_activate (GtkMenuItem * menuitem, gpointer user_data)
{

	GtkWidget *dialog_about=NULL;

	dialog_about = create_dialog_about ();

	gtk_dialog_run (GTK_DIALOG (dialog_about));
	gtk_widget_destroy (GTK_WIDGET (dialog_about));
}

void
on_button_browseoutput_clicked (GtkButton * button, gpointer user_data)
{
	GtkWidget *filechooserdialog_output=NULL;

	filechooserdialog_output = gtk_file_chooser_dialog_new ("Open File", NULL,	/*parent_window, */
								GTK_FILE_CHOOSER_ACTION_OPEN,
								GTK_STOCK_CANCEL,
								GTK_RESPONSE_CANCEL,
								GTK_STOCK_OPEN,
								GTK_RESPONSE_ACCEPT,
								NULL);

	if (gtk_dialog_run (GTK_DIALOG (filechooserdialog_output)) ==
	    GTK_RESPONSE_ACCEPT)
	{
		char *filename;

		filename =
			gtk_file_chooser_get_filename (GTK_FILE_CHOOSER
						       (filechooserdialog_output));

		gtk_entry_set_text (GTK_ENTRY
				    (lookup_widget
				     (GTK_WIDGET (button),
				      "entry_outputfile")), filename);

		g_free (filename);

	}

	gtk_widget_destroy (filechooserdialog_output);

}

gboolean
on_window1_delete_event (GtkWidget * widget,
			 GdkEvent * event, gpointer user_data)
{

	if (quit_confirm ())
		gtk_main_quit ();

	return TRUE;
}

void
on_quit1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	if (quit_confirm ())
		gtk_main_quit ();
}

gboolean
quit_confirm ()
{
	GtkWidget *dialog_quit=NULL;
	gboolean quit;

	dialog_quit = create_dialog_quit ();

	quit = (gtk_dialog_run (GTK_DIALOG (dialog_quit)) ==
		GTK_RESPONSE_ACCEPT);

	gtk_widget_destroy (dialog_quit);

	return quit;
}

void
on_button_exec_clicked (GtkButton * button, gpointer user_data)
{
	gchar *errMsg = NULL;
	GtkWindow *dialog=NULL;
	
	errMsg = execute_ok (GTK_WIDGET (button));

	if (errMsg)
	{
		dialog = GTK_WINDOW(create_dialog_err(errMsg));
		gtk_dialog_run(GTK_DIALOG(dialog));
		gtk_widget_destroy(GTK_WIDGET(dialog));
	}
	else
		run_report ();
}


void
on_button_param_clicked (GtkButton * button, gpointer user_data)
{
	xmlDoc *doc=NULL;
	GList *walk = NULL;
	gchar *filename=NULL, *fstr=NULL;
	FILE *handle=NULL;
	require_parameter *p = NULL;
	GList *parameter_list_new = NULL;
	GtkWidget *dialog_param = NULL;
	
	filename = mem_strdup (gtk_entry_get_text (GTK_ENTRY (lookup_widget (GTK_WIDGET (button), "entry_report_filename"))));
	
	g_strstrip(filename);
	
	if (!filename[0])	/* || strlen(filename)) */
	{
		warning_dialog ("No filename entered!");
		return;
	}

	doc = parse_xml_doc (filename);
	if (!doc)
		return;

	param_getfromdoc (doc, &parameter_list_new);
	printf("merge:\n");
	param_list_merge (&parameter_list, &parameter_list_new);
	
	printf ("merged, freeing old p list\n");
	for (walk = g_list_first (parameter_list); walk; walk = walk->next)
	{
			printf ("*free*\n");
			p = walk->data;
			g_free (p->name);
			g_free (p->description);
	}
		g_list_free (parameter_list);
		printf ("freed\nNew param list:\n");
		parameter_list = parameter_list_new;

	for (walk = g_list_first (parameter_list); walk != NULL;
	     walk = walk->next)
	{
		p = walk->data;
		printf ("Parameter: name=%s description=%s\n", p->name,
			p->description);
	}

	printf ("got param_list \n");
	
	dialog_param = create_dialog_param();
	gtk_widget_show(dialog_param);
}

void
on_combobox_outputtypes_changed (GtkComboBox * combobox, gpointer user_data)
{
	GtkWidget *entry_output=NULL;
	gchar *outfile=NULL, *outfilenew=NULL, *ext = NULL;
	int active;

	entry_output =
		lookup_widget (GTK_WIDGET (combobox), "entry_outputfile");

	outfile = mem_strdup (gtk_entry_get_text (GTK_ENTRY (entry_output)));
	active = gtk_combo_box_get_active (combobox);

	if (active == -1 || !outfile[0] || isspace (outfile[0]))
		return;

	ext = g_list_nth_data (outputtypes, active);
	/*/printf ("outfile: %s, ext: %s\n", outfile, ext);*/
	outfilenew = change_extension (outfile, ext);
	if (outfilenew)
	{
		gtk_entry_set_text (GTK_ENTRY (entry_output), outfilenew);
		mem_free (outfilenew);
	}
	else
		gtk_entry_set_text (GTK_ENTRY (entry_output), outfile);
}

void
on_param_ok_clicked 					(GtkButton *button,
										gpointer	user_data)
{	
	GList *walk = NULL;
	GtkWidget *dialog_param=NULL;

	dialog_param = lookup_widget(GTK_WIDGET(button), "dialog_param");

	param_store_list();
	param_delete_widgets();
	
	gtk_widget_destroy(dialog_param);
}

void
on_param_cancel_clicked 					(GtkButton *button,
										gpointer	user_data)
{
	GtkWidget *dialog_param=NULL;
	
	dialog_param = lookup_widget(GTK_WIDGET(button), "dialog_param");
	gtk_widget_destroy(dialog_param);
}

void
on_savefav_activate (GtkMenuItem * menuitem, gpointer user_data)
{
	GtkWindow *dialog=NULL;

	dialog = GTK_WINDOW(create_dialog_err(mem_strdup("Not yet implemented")));
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(GTK_WIDGET(dialog));
}

void
on_entry_report_filename_changed (GtkEditable * editable, gpointer user_data)
{
	GtkWidget *button_param=NULL;
	GtkWidget *menu_param=NULL;
	
	button_param = lookup_widget (GTK_WIDGET (editable), "button_param");
	menu_param = lookup_widget (GTK_WIDGET (editable), "parameters");
	
	if (!param_ok (GTK_WIDGET (editable)))
	{
		gtk_widget_set_sensitive (button_param, TRUE);
		gtk_widget_set_sensitive (menu_param, TRUE);
	}
	else
	{
		gtk_widget_set_sensitive (button_param, FALSE);
		gtk_widget_set_sensitive (menu_param, FALSE);
	}
}
