#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/stat.h>
#include <sys/types.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <glib.h>
#include <gtk/gtk.h>

#include "bc.h"

#include "init.h"

#include "support.h"

extern gchar *config_path;
extern gchar *fav_file;
extern GtkWidget *window1;

/* return TRUE on success */
gboolean
init_populate_favourites(gchar *favourites_file, GtkWidget **treeview)
{
	FILE *file=NULL;
	int row=0;
	gchar buf[256];
	GtkListStore *store_favourites=NULL;
	GtkTreeIter iter_fav;
	
	gchar *name=NULL, *path=NULL;
	
	g_assert(favourites_file);
	g_assert(treeview);
	
	if(!(file = fopen(favourites_file, "r")))
	{
		fprintf(stderr, "Could not open favourites file: %s\n", favourites_file);
		return FALSE;
	}	
	
	store_favourites = gtk_list_store_new (3, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING);
	gtk_tree_view_set_model(GTK_TREE_VIEW(*treeview), GTK_TREE_MODEL(store_favourites));

	while(fgets(buf, 255, file))
	{
		if(buf[0]=='#' || isspace(buf[0]))
			continue;
	
		name=mem_strdup(buf);
		
		/* get next line (path) */
		if(!fgets(buf, 255, file))
			break;

		path=mem_strdup(buf);
		/* fill columns */
		gtk_list_store_append(store_favourites, &iter_fav);
		gtk_list_store_set (store_favourites, &iter_fav,	
														0, row, 
														1, name,
														2, path,
														-1);
		printf("added row=%d name=%s path=%s\n", row, name, path);
		gtk_tree_model_row_inserted(GTK_TREE_MODEL(store_favourites), 
									gtk_tree_path_new_from_string(mem_strdup_printf("0%d", row)),
									&iter_fav);
		gtk_tree_model_row_inserted(GTK_TREE_MODEL(store_favourites), 
									gtk_tree_path_new_from_string(mem_strdup_printf("1%d", row)),
									&iter_fav);
		gtk_tree_model_row_inserted(GTK_TREE_MODEL(store_favourites), 
									gtk_tree_path_new_from_string(mem_strdup_printf("2%d", row)),
									&iter_fav);
		row++;
	}
	
	fclose(file);
	
	return TRUE;
}

gboolean init_setup(int argc, char *argv[])
{
	GtkWidget *treeview_favourites=NULL;

	GtkTreeViewColumn *tree_view_column_hidden=NULL;
	GtkTreeViewColumn *tree_view_column_name=NULL;
	GtkTreeViewColumn *tree_view_column_path=NULL;

	int tmp=0;
	gchar *home=NULL, *pos=NULL, *temp=NULL;
	
	home = getenv("HOME");
	
	config_path = mem_strdup(bcconfig_getvalue("papyrus_config_path"));

	/* replace '~' (=126) with $HOME */
	pos=strchr(config_path, 126);
	if(pos)
	{
		temp = mem_strdup_printf("%s%s", home, &(pos[1]));
		mem_free(config_path);
		config_path=temp;
	}
	
	if(argv[1])
		fav_file = mem_strdup(argv[1]);
	else
		fav_file = mem_strdup_printf("%s/%s", 
			config_path,
			bcconfig_getvalue("favourites_file"));

	pos=strchr(fav_file, 126);
	if(pos)
	{
		temp = mem_strdup_printf("%s%s", home, &(pos[1]));
		mem_free(fav_file);
		fav_file=temp;
	}
	if(!g_file_test(config_path, G_FILE_TEST_EXISTS))
	{
		tmp=mkdir(config_path, 0755);
		printf("creating directory %s returned %d\n", config_path, tmp);
	}
	treeview_favourites = lookup_widget(window1, "treeview_favourites");
	
	tree_view_column_name = gtk_tree_view_column_new ();
	tree_view_column_path = gtk_tree_view_column_new ();
	tree_view_column_hidden = gtk_tree_view_column_new ();
	
	gtk_tree_view_insert_column(GTK_TREE_VIEW(treeview_favourites), GTK_TREE_VIEW_COLUMN(tree_view_column_hidden), 0);
	gtk_tree_view_insert_column(GTK_TREE_VIEW(treeview_favourites), GTK_TREE_VIEW_COLUMN(tree_view_column_name), 0);
	gtk_tree_view_insert_column(GTK_TREE_VIEW(treeview_favourites), GTK_TREE_VIEW_COLUMN(tree_view_column_path), 0);
	
	init_populate_favourites(fav_file, &treeview_favourites);
	
	return TRUE;
}
