#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <libxml/xmlreader.h>
#include <gtk/gtk.h>
#include <time.h>

#include "bc.h"

#include "report.h"

#include "callbacks.h"
#include "interface.h"
#include "support.h"

gboolean run_report()
{
	gchar *cmd=NULL, *ext=NULL, *outfile=NULL, *infile=NULL;
	GtkWidget *entry_output=NULL, *entry_report_filename=NULL;
	GtkWidget *combobox=NULL, *checkbutton_openviewer=NULL;
	GList *walk=NULL;
	require_parameter *martin=NULL;	
	int outputtype_active=0;
	
	cmd = bcconfig_getvalue("report_binary");
	
	if(parameter_list)
	{
		walk = g_list_first(parameter_list);
		while(walk)
		{
			martin = (require_parameter*)walk->data;
			
			if(martin->value)
				cmd = mem_strdup_printf("%s -a %s=%s", cmd, 
					martin->name, martin->value);
			
			walk = walk->next;
		}
	}

	entry_output = lookup_widget (GTK_WIDGET (window1), "entry_outputfile");
	entry_report_filename = lookup_widget (GTK_WIDGET (window1), "entry_report_filename");
	combobox = lookup_widget (GTK_WIDGET(window1), "combobox_outputtypes");
	checkbutton_openviewer = lookup_widget (GTK_WIDGET(window1), "checkbutton_openviewer");
	
	outfile = g_strstrip(mem_strdup (gtk_entry_get_text (GTK_ENTRY (entry_output))));
	infile = g_strstrip(mem_strdup (gtk_entry_get_text (GTK_ENTRY (entry_report_filename))));
	outputtype_active = gtk_combo_box_get_active (GTK_COMBO_BOX(combobox));

	if (outputtype_active == -1 || !outfile[0] || isspace (outfile[0]))
		return FALSE;

	ext = g_list_nth_data (outputtypes, outputtype_active);
	
	cmd = mem_strdup_printf("%s -t %s -o %s", cmd, g_ascii_strdown(ext, -1), outfile);
	
	cmd = mem_strdup_printf("%s %s", cmd, infile);
	cmd = mem_strdup_printf("%s -d", cmd);
	
	if(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(checkbutton_openviewer))
		==TRUE)
		cmd = mem_strdup_printf("%s -v", cmd); 

	printf("Running report\n%s\n", cmd);
	system(cmd);
	printf("Finished\n");
	
	/*	
	dialog = create_dialog_err(cmd);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(GTK_WIDGET(dialog));
	*/
	
	mem_free(cmd);
	mem_free(infile);
	mem_free(outfile);
	
	return 0;
	
	/*
		int g = 0;
	char *args[64];
	
	cmd = bcconfig_getvalue("report_binary");
	
	if(parameter_list)
	{
		walk = g_list_first(parameter_list);
		while(walk)
		{
			martin = (require_parameter*)walk->data;
			
			if(martin->value)
			{
				args[g++] = mem_strdup("-a");
				args[g++] = mem_strdup_printf("%s=%s", 
					martin->name, martin->value);
			}
			walk = walk->next;
		}
	}

	entry_output = lookup_widget (GTK_WIDGET (window1), "entry_outputfile");
	entry_report_filename = lookup_widget (GTK_WIDGET (window1), "entry_report_filename");
	combobox = lookup_widget (GTK_WIDGET(window1), "combobox_outputtypes");
	checkbutton_openviewer = lookup_widget (GTK_WIDGET(window1), "checkbutton_openviewer");
	
	outfile = g_strstrip(mem_strdup (gtk_entry_get_text (GTK_ENTRY (entry_output))));
	infile = g_strstrip(mem_strdup (gtk_entry_get_text (GTK_ENTRY (entry_report_filename))));
	outputtype_active = gtk_combo_box_get_active (GTK_COMBO_BOX(combobox));

	if (outputtype_active == -1 || !outfile[0] || isspace (outfile[0]))
		return;

	ext = g_list_nth_data (outputtypes, outputtype_active);
	
	args[g++] = mem_strdup("-t");
	args[g++] = mem_strdup(g_ascii_strdown(ext, -1));
	args[g++] = mem_strdup("-o");
	args[g++] = mem_strdup(outfile);
	
	args[g++] = mem_strdup(infile);
	
	args[g++] = mem_strdup("-d");
	
	if(gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(checkbutton_openviewer))
		==TRUE)
		args[g++] = mem_strdup("-v");
	
	args[g] = NULL;
	
	exec_return = execvp(cmd, args);
	*/
}

/**
	execute_ok
	returns reason why report can't be executed or NULL if it can
*/
gchar *
execute_ok(GtkWidget *start)
{
	gchar *retval = NULL;
	gchar *tmp=NULL;
	
	g_assert(start);
	
	retval = mem_strdup("");

	/* check if parameters are filled in */
	if(!parameter_list)
		retval = mem_strdup_printf("%sEnter parameters before running report\n", retval);

	tmp = param_ok(start);
	if(tmp)
		retval = mem_strdup_printf("%s%s", retval, tmp);
	
	
	if(strlen(retval))
		return retval;
	else 
		return NULL;
}
/* checking to see if there is sufficient *
 * information to open parameter window   */
gchar *
param_ok(GtkWidget *start)
{
	gchar *retval = NULL;
	gchar *tmp = NULL;
	GtkWidget *entry_report_filename = NULL;
	GtkWidget *entry_outputfile = NULL;

	g_assert(start);
		
	/* check input and output fields have values */
	entry_report_filename = lookup_widget(start, "entry_report_filename");
	entry_outputfile = lookup_widget(start, "entry_outputfile");

	retval = mem_strdup("");

	tmp = mem_strdup(gtk_entry_get_text(GTK_ENTRY(entry_report_filename)));
	g_strstrip(tmp);
	if(!strlen(tmp))
		retval = mem_strdup_printf("%sEnter report filename\n", retval);

	tmp = mem_strdup(gtk_entry_get_text(GTK_ENTRY(entry_outputfile)));
	g_strstrip(tmp);
	if(!strlen(tmp))
		retval = mem_strdup_printf("%sEnter output filename\n", retval);

	if(strlen(retval))
		return retval;
	else 
		return NULL;	
}

xmlDoc *
parse_xml_doc(gchar * xmlfilename)
{
  xmlDoc *doc = NULL;
  gchar *err= NULL;
  xmlTextReaderPtr reader = NULL;
  int ret;

	g_assert(xmlfilename);
	
  xmlInitParser();
  reader = xmlReaderForFile(xmlfilename, NULL, 0);
  if (reader != NULL) {
	ret = xmlTextReaderRead(reader);
	while (ret == 1) {
		  //processNode(reader);
	  ret = xmlTextReaderRead(reader);
	}
	xmlFreeTextReader(reader);
	if (ret != 0) {
	  err = mem_strdup_printf("%s : Failed to parse\n", xmlfilename);
	}
  } else {
	err = mem_strdup_printf("%s : Unable to open\n", xmlfilename);
  }

  if(err)
	{
	  fprintf(stderr, err);
	  
	  //	  xmlFreeDoc(doc);
	  warning_dialog(err);
	  return NULL;
	}
  doc = xmlParseFile(xmlfilename);

  return doc;
}

/**
  * param_getfromdoc: gets the required fields for the report
  * <requires>blah</requires>
  * returns: GList of parameters
  */
void
param_getfromdoc(xmlDoc *doc, GList **param_list)
{
  xmlNode *node=NULL;
  
	g_assert(doc);
	g_assert(param_list);
	
  node = xmlDocGetRootElement(doc);
  
  param_addtolist(node, param_list);
}

static void
param_addtolist(xmlNode * a_node, GList **list)
{
  require_parameter *param = NULL;
  xmlNode *cur_node = NULL;
  gchar * tmp = NULL;
  
	g_assert(a_node);
	g_assert(list);

	for (cur_node = a_node; cur_node; cur_node = cur_node->next) {
	if ((cur_node->type == XML_ELEMENT_NODE) && 
		(!strcmp("requires", cur_node->name)))
	  {
		param = mem_calloc(sizeof(require_parameter));
		param->name = NULL;
		param->description = NULL;
		param->type = 0;

		tmp = xmlNodeGetContent(cur_node);
		if(tmp)
		  param->name = mem_strdup(tmp);
		else 
		  continue;
		tmp = xmlGetProp(cur_node, "description");
		if(tmp)
		  param->description = mem_strdup(tmp);
		else
		  param->description = mem_strdup(param->name);
		
		tmp = xmlGetProp(cur_node, "type");
		if(tmp)
		  param->type = param_type_from_str(tmp);
		
		printf("adding %s, desc=%s, type=%d\n", param->name, param->description, param->type);
		list = g_list_append(*list, param);
		//printf("added\n");
	  }
	if(cur_node->children)
	  param_addtolist(cur_node->children, list);
  }
}

gchar *
change_extension(gchar *oldfilename, gchar *ext)
{
  gchar *newfilename = NULL;
  int slen, i;

  g_assert(oldfilename);
	
  if (!ext)
	  return NULL;
  slen = strlen(oldfilename);
  for (i=slen-1;i>0;i--)
	  if (oldfilename[i]=='.')
		{
		oldfilename[i] = 0;
		break;
	  	}
  
  
  newfilename = mem_strdup_printf("%s.%s",oldfilename,g_ascii_strdown(ext, -1));
  
  /*printf("newfilename: %s, ext: %s len: %d\n", 
		 newfilename, ext, strlen(newfilename));*/
  
  return newfilename;
}

/* requires 2 x Glist of requires_param
 * if param exists in both, copy value to new list
 */
void
param_list_merge(GList **list_cur, GList **list_new)
{
	GList *walk_cur = NULL, *walk_new = NULL;
	require_parameter *p_cur = NULL, *p_new = NULL;

	g_assert(list_cur);
	g_assert(list_new);	

	/* 	iterate thru list_new checking if we want to copy value
	 *	from list_cur
	*/
	walk_new = g_list_first(*list_new);
	while(walk_new)
	{
		p_new = walk_new->data;
		
		walk_cur = g_list_first(*list_cur);
		while(walk_cur)
		{
			p_cur = walk_cur->data;

			if(p_new->name && p_cur->name && !strcmp(p_new->name, p_cur->name))
			{
				printf("copying value %s=%s\n", p_cur->name, p_cur->value);
				p_new->value = mem_strdup(p_cur->value);
			}
			else
			{
				printf("discarding value for %s\n", p_cur->name);
			}
			 walk_cur = walk_cur->next;
		}
		 walk_new = walk_new->next;
	}
}

int param_type_from_str(gchar *str)
{
	if(!str)
		return TYPE_TEXT; // text
	if(!strcmp(str, "int"))
		return TYPE_INT;
	if(!strcmp(str, "date"))
		return TYPE_DATE;
	if(!strcmp(str, "dropdown"))
		return TYPE_DROPDOWN;

	return TYPE_TEXT;
}

int param_store_list()
{
	GList *walk = NULL;
	require_parameter *p = NULL;
	guint year=0, month=0, day=0;
	
	printf("storing list...\n");
	for(walk=g_list_first(parameter_list); walk; walk=walk->next)
	{
		p = walk->data;
		switch(p->type)
		{
			case TYPE_DATE:
				gtk_calendar_get_date(GTK_CALENDAR(p->widget), &year, &month, &day);
				month++; /* (JAN = 0, DEC = 11) */
				if(year && month && day)
					p->value = mem_strdup_printf("%d-%02d-%02d", year, month, day);
				else p->value = NULL;
				break;
			case TYPE_INT:
				p->value = mem_strdup_printf("%d", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(p->widget)));
				break;
			case TYPE_DROPDOWN:
				/* TODO */
				p->value = NULL;
				break;
			case TYPE_TEXT:
			default:
				p->value = mem_strdup(gtk_entry_get_text(GTK_ENTRY(p->widget)));
		}
	}
	return 0;
}

int date_from_str(char *str, int *year, int *month, int *day)
{
		time_t t;
		struct tm tm;
		char buf[255];
		
		g_assert(str);
		
		t = time(0);
		memset(&tm,0,sizeof(struct tm));
		memcpy(&tm,localtime(&t),sizeof(struct tm));
		if(!strptime(str, "%Y-%m-%d", &tm))
		return -1;

		strftime(buf, sizeof(buf), "%d", &tm);
		*day = atoi(buf);
		strftime(buf, sizeof(buf), "%m", &tm);
		*month = atoi(buf);
		strftime(buf, sizeof(buf), "%Y", &tm);
		*year = atoi(buf);
		
		return 0;
}
