##       Author:  Neil Prockter  <prockter@lse.ac.uk>
##      Created:  24/01/04 16:38:25 GMT

=head1 license

This file is part of Synaptic.

Synaptic is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Synaptic is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Synaptic; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

=cut

#### vars from cvs keyword
my $ID = q$Id: NagiosCGIApp.pm,v 1.18 2004/12/09 00:21:03 neil Exp $ ;
my $version = join (' ', (split (' ', $ID))[2]);

## in a file called NagiosCGIApp.pm
package NagiosCGIApp ;
use base 'CGI::Application' ;
use strict ;

## override (should be elsewhere)
sub load_tmpl {
  my $self = shift;
  my ($tmpl_file, @extra_params) = @_;

  # add tmpl_path to path array of one is set, otherwise add a path arg
  if (my $tmpl_path = $self->tmpl_path) {
    my $found = 0;
    for( my $x = 0; $x < @extra_params; $x += 2 ) {
      if ($extra_params[$x] eq 'path' and ref $extra_params[$x+1] and ref $extra_params[$x+1] eq 'ARRAY') {
        unshift @{$extra_params[$x+1]}, $tmpl_path;
        $found = 1;
        last;
      }
    }
    push(@extra_params, path => [ $tmpl_path ]) unless $found;
  }

  require HTML::Template::Expr;
  my $t = HTML::Template::Expr->new_file($tmpl_file, @extra_params);

  return $t;
}



## remove these when live
use warnings ;
use diagnostics ;
use CGI::Carp ;

## handy things
use Data::Dumper ;
#use FindBin ;
#use lib "$FindBin::Bin/lib" ;

use Nagios::Config ;
use Nagios::StatusLog ;
use HTML::Template::Expr ;

use Date::Manip ;


#### these should in a common library used here for counters
my %global_service_totals = (
  'OK'       => 0,
  'WARNING'  => 0,
  'CRITICAL' => 0,
  'UNKNOWN'  => 0,
);

#####################################################
my $debug = '' ;
my $rowid = 0 ;


sub setup {
  my $self = shift;
  $self->start_mode('status');
  $self->run_modes(
      'status' => 'status_report',
      'oldstatus' => 'old_style_status_report',
      'version' => 'show_versions',
  );
};



sub rowincr
{
  $rowid++ ;
  return '' ;
}

sub status_bg_class
{
  my $status = shift ;
  #return "$rowid: $host, $service" ;
  return "BG$status" unless $status eq 'OK' ;
  return ( $rowid % 2 ) ? 'Even' : 'Odd' ;
}

sub DateManipUnixDateEpoch
{
  my $epoch = shift ;
  my $time = &UnixDate( &ParseDateString("epoch $epoch"), "%Y-%m-%d %H:%M:%S" ) ;
  return $time ;
}

sub DateManipDateCalcEpoch
{
  my $epoch = shift ;
  my $today = &ParseDate( "today" ) ;
  my $delta = &DateCalc( "epoch $epoch", $today );
  my $time = &Delta_Format( $delta, 0, "%dhd %hvh %mvm %svs"  ) ;
  return $time ;
} 
#print DateManipDateCalcEpoch( 1073829541 ), "\n" ;

sub show_versions
{
  return $version ;
}

#####################################################

sub status_report
{
  my $webapp = shift ;
  
  ## page should refresh
  $webapp->header_add( -refresh => 15 );
  
  ## read input parameters  
  my $q = $webapp->query();

  $webapp->param( 'debug', 0 ) ;
  
  ## style - which templates to use - default or classic at present
  if ( defined $q->param( 'style' ) ) {
    $webapp->param( 'style', $q->param('style') ) ;
  }

  ## whether to show ok services
  $webapp->param( 'showok', 0 ) ;
  if ( defined $q->param( 'showok' ) ) {
    $webapp->param( 'showok', 1 ) ;
  }
  
  my $template ;
  eval {
    $template = $webapp->load_tmpl(
      sprintf( "%s/status.html.tmpl", $webapp->param('style') ) ,
      'die_on_bad_params' => 0 ,
      'loop_context_vars' => 1 ,
      'functions' => {
        'status_bg_class' => \&status_bg_class ,
        'rowincr' => \&rowincr ,
        'DateManipUnixDateEpoch' => \&DateManipUnixDateEpoch ,
        'DateManipDateCalcEpoch' => \&DateManipDateCalcEpoch ,
      } ,
    ) ;
  };
  if ( $@ ) {
    $debug .= "Error opening template $@\n" ;
    return "<html><body><pre>$debug</pre></body></html>" ;
  }
 
  my $config ;
  eval {
    $config = Nagios::Config->new( 'Filename' => '/etc/nagios/nagios.cfg' );
  } ;
  if ( $@ ) {
    $debug .= "Error opening config $@\n" ;
    $template->param(
      'running' => 0,
      'message' => $@,
      'debug' => $debug ,
    );
    return $template->output() ;
  }
 
  my $log ;
  eval {
    $log = Nagios::StatusLog->new( '/var/log/nagios/status.log' ) ;
  } ;
  if ( $@ ) {
    $debug .= "Error opening status.log $@\n" ;
    $template->param(
      'running' => 0,
      'message' => $@,
      'debug' => $debug ,
    );
    return $template->output() ;
  }
 
  #$debug .= 'Log ' . Dumper( \$log ) ;
  #$debug .= 'Config ' . Dumper( \$config ) ;

  my @hosts ;
  my @problem_hosts ;
HOST: for my $host ( $config->list_hosts ) {
    next if ( !length $host->host_name ); # avoid a bug in Nagios::Object
    $debug .= sprintf( "HOST: %s\n", $host->host_name ) if $webapp->param( 'debug' ) ;
    my $all_ok = 1 ;
    my @services ;
SERVICE: for my $servicename ( $host->list_services ) {
      my $service = $log->service( $host, $servicename );
      $debug .= sprintf( "SERVICE: %s %s\n", $servicename->service_description, Dumper( $service ) ) if $webapp->param( 'debug' ) ;
      $global_service_totals{ $service->status }++ ;
      push @services, $service ;
      if ( $service->status ne 'OK' ) {
        $all_ok = 0 ;
      }
    }
    $global_service_totals{'TOTAL'} += $#services + 1 ;
    @services = sort byPluginThenDescription @services ;
    if ( $all_ok == 1 )
    {
      push @hosts, { 
        'hostname' => $host->host_name ,
        'services' => \@services ,
      } ;
    } else {
      push @problem_hosts, { 
        'hostname' => $host->host_name ,
        'services' => \@services ,
      } ;
    }
  }

  $debug .= '%global_service_totals ' . Dumper( \%global_service_totals ) ; 

  @problem_hosts = sort byName @problem_hosts ;

  ## if showing all ok then sort else randomly choose 1 to show
  if ( $webapp->param( 'showok' ) ) {
    @hosts = sort byName @hosts ;
  } else {
    @hosts = splice @hosts, int(rand( $#hosts )), 1 ;
  }

  $template->param(
    'running' => 1,
    'problemhosts' => \@problem_hosts ,
    'hosts' => \@hosts ,
    'service_totals' => $global_service_totals{'TOTAL'} ,
    'service_totals_ok' => $global_service_totals{'OK'} ,
    'service_totals_warning' => $global_service_totals{'WARNING'} ,
    'service_totals_critical' => $global_service_totals{'CRITICAL'} ,
    'service_totals_problems' => $global_service_totals{'WARNING'} + $global_service_totals{'CRITICAL'} ,
    'debug' => ( $webapp->param( 'debug' ) eq 1 ? $debug : 'debug off' ) ,
  ) ;

  return $template->output() ;
}

#-----------------------------------------------------------------------
#  sorting functions
#-----------------------------------------------------------------------
sub byPluginThenDescription {
  return $a->{'description'} cmp $b->{'description'} ;
}

sub byName {
  return $a->{'hostname'} cmp $b->{'hostname'} ;
}

1;  # Perl requires this at the end of all modules


__END__
 vim:tw=100:ts=8:ft=perl:norl:
